@@ -1,23 +1,20 @@
+/**************************************************************************
+*   Copyright (C) 2005-2012 by Oleksandr Shneyder                         *
+*   o.shneyder@phoca-gmbh.de                                              *
+*                                                                         *
+*   This program is free software; you can redistribute it and/or modify  *
+*   it under the terms of the GNU General Public License as published by  *
+*   the Free Software Foundation; either version 2 of the License, or     *
+*   (at your option) any later version.                                   *
+*   This program is distributed in the hope that it will be useful,       *
+*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+*   GNU General Public License for more details.                          *
+*                                                                         *
+*   You should have received a copy of the GNU General Public License     *
+*   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
+***************************************************************************/
 
-/***************************************************************************
- *   Copyright (C) 2005-2012 by Oleksandr Shneyder                         *
- *   oleksandr.shneyder@obviously-nice.de                                  *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
- ***************************************************************************/
 #ifndef ONMAINWINDOW_H
 #define ONMAINWINDOW_H
 
@@ -40,9 +37,11 @@
 #include <QPushButton>
 #include <QPixmap>
 #include <QProcess>
+#include <QTreeView>
 #include "LDAPSession.h"
 #include <QToolBar>
 #include <QSystemTrayIcon>
+#include "sshmasterconnection.h"
 
 
 #ifdef Q_OS_WIN
@@ -72,9 +71,7 @@ class SVGFrame;
 class SessionButton;
 class QAction;
 class QCheckBox;
-class QTreeView;
 class QModelIndex;
-class SshProcess;
 class SshMasterConnection;
 class IMGFrame;
 class QStandardItemModel;
@@ -99,7 +96,7 @@ struct directory
     QString dstKey;
     QString dirList;
     bool isRemovable;
-    SshProcess* proc;
+    int pid;
 };
 
 struct serv
@@ -164,12 +161,15 @@ struct ConfigFile
     QString brokerPass;
     QString brokerUserId;
     QString brokerName;
+    QString sshBrokerBin;
     bool brokerAuthenticated;
+    bool brokerNoAuth;
+    bool brokerAutologin;
+    QString brokerSshKey;
     QString iniFile;
     QString server;
+    QString serverIp;//Can be different from server (use for loadballancing)
     QString sshport;
-    QString proxy;
-    QString proxyport;
     QString command;
     QString key;
     bool rootless;
@@ -203,14 +203,24 @@ struct ConfigFile
     QString kbdLay;
     QString kbdType;
     //
+    bool useproxy;
+    SshMasterConnection::ProxyType proxyType;
+    QString proxyserver;
+    int proxyport;
+    QString proxylogin;
+    QString proxypassword;
+    QString proxyKey;
+    bool proxyAutologin;
+
 };
 
-struct SshProxy
+
+struct sshKey
 {
-    bool use;
-    QString host;
+    QString server;
     QString port;
-    QString bin;
+    QString user;
+    QString key;
 };
 
 
@@ -234,13 +244,33 @@ private:
     ONMainWindow* parent;
 };
 #endif
+
+
+class SessTreeView : public QTreeView
+{
+    Q_OBJECT
+
+public:
+    SessTreeView ( QWidget* parent = 0 ) : QTreeView ( parent ) {}
+
+    virtual void selectionChanged ( const QItemSelection& selected,
+                                    const QItemSelection& deselected ) {
+        emit this->selected ( currentIndex() );
+        QTreeView::selectionChanged ( selected, deselected );
+    }
+
+Q_SIGNALS:
+    void selected ( const QModelIndex& index );
+};
+
+
 class ClickLineEdit;
 class ONMainWindow : public QMainWindow
 #ifdef CFGPLUGIN
-            , public QtNPBindable
+    , public QtNPBindable
 
 #ifdef QAXSERVER
-            , public QAxBindable
+    , public QAxBindable
 #endif
 #endif
 {
@@ -297,11 +327,13 @@ public:
     static bool portable;
     ONMainWindow ( QWidget *parent = 0 );
     ~ONMainWindow();
+    static void installTranslator();
     QString iconsPath ( QString fname );
     const QList<SessionButton*> * getSessionsList()
     {
         return &sessions;
     }
+    static bool isServerRunning ( int port );
     void startNewSession();
     void suspendSession ( QString sessId );
     bool termSession ( QString sessId,
@@ -367,6 +399,14 @@ public:
     {
         return config.showextconfig;
     }
+    bool getUsePGPCard()
+    {
+        return usePGPCard;
+    }
+    QString getCardLogin()
+    {
+        return cardLogin;
+    }
     QString getDefaultCmd()
     {
         return defaultCmd;
@@ -473,9 +513,7 @@ public:
     static QString cygwinPath ( const QString& winPath );
     void startXOrg();
     void startPulsed();
-    static bool haveCySolEntry();
     static bool haveCygwinEntry();
-    static void removeCySolEntry();
     static void removeCygwinEntry();
     static QString U3DevicePath()
     {
@@ -488,6 +526,8 @@ private:
     QStringList _internApplicationsNames;
     QStringList _transApplicationsNames;
     QString portableDataPath;
+    QString proxyErrString;
+    bool proxyRunning;
     bool drawMenu;
     bool extStarted;
     bool startMaximized;
@@ -507,6 +547,7 @@ private:
     bool thinMode;
     QString statusString;
     QString autostartApp;
+    bool cmdAutologin;
     int defaultLink;
     int defaultQuality;
     int defaultWidth;
@@ -520,7 +561,7 @@ private:
     bool noSessionEdit;
     bool cleanAllFiles;
     bool PGPInited;
-    struct SshProxy sshProxy;
+    bool resumeAfterSuspending;
     QString sshPort;
     QString clientSshPort;
     QString defaultSshPort;
@@ -545,6 +586,7 @@ private:
     int retSessions;
     QList<serv> x2goServers;
     QList<Application> applications;
+    QList<sshKey> cmdSshKeys;
 
     QPushButton* bSusp;
     QPushButton* bTerm;
@@ -555,7 +597,7 @@ private:
 
 
     QLabel* selectSessionLabel;
-    QTreeView* sessTv;
+    SessTreeView* sessTv;
 
     QLineEdit* desktopFilter;
     QCheckBox* desktopFilterCb;
@@ -669,7 +711,9 @@ private:
     bool ldapOnly;
     bool isScDaemonOk;
     bool parecTunnelOk;
-
+#ifdef Q_OS_LINUX
+    bool directRDP;
+#endif
 
     bool startSessSound;
     int startSessSndSystem;
@@ -713,6 +757,8 @@ private:
     static QString u3Device;
 
     QProcess* pulseServer;
+    QStringList pulseArgs;
+    QTimer* pulseTimer;
     int xDisplay;
     int sshdPort;
     bool winServersReady;
@@ -720,7 +766,6 @@ private:
     QString oldBinDir;
     QString oldTmpDir;
 
-    bool cySolEntry;
     bool cyEntry;
 
     QString pulseDir;
@@ -738,9 +783,9 @@ private:
     QTextEdit* stInfo;
 
     SVGFrame* ln;
-    SshProcess* tunnel;
-    SshProcess* sndTunnel;
-    SshProcess* fsTunnel;
+    int tunnel;
+    int sndTunnel;
+    int fsTunnel;
     QList<x2goSession> selectedSessions;
     QStringList selectedDesktops;
     x2goSession resumingSession;
@@ -792,7 +837,7 @@ private:
     bool trayMaxDiscon;
     bool trayAutoHidden;
 
-    void installTranslator();
+    QString findSshKeyForServer(QString user, QString server, QString port);
     void loadSettings();
     void showPass ( UserButton* user );
     void clean();
@@ -835,7 +880,11 @@ private:
     void continueLDAPSession();
     SshMasterConnection* startSshConnection ( QString host, QString port,
             bool acceptUnknownHosts, QString login,
-            QString password, bool autologin, bool krbLogin, bool getSrv=false);
+            QString password, bool autologin, bool krbLogin, bool getSrv=false, bool useproxy=false,
+            SshMasterConnection::ProxyType type=SshMasterConnection::PROXYSSH,
+            QString proxyserver=QString::null, quint16 proxyport=0,
+            QString proxylogin=QString::null, QString proxypassword=QString::null, QString proxyKey=QString::null,
+            bool proxyAutologin=false );
     void setProxyWinTitle();
     QRect proxyWinGeometry();
     void readApplications();
@@ -856,6 +905,8 @@ private slots:
     void startWinServers();
     void slotCheckXOrgLog();
     void slotCheckXOrgConnection();
+    void slotCheckPulse();
+    void slotStartParec ();
 #endif
 private slots:
     void slotAppDialog();
@@ -878,14 +929,15 @@ private slots:
     void displayToolBar ( bool );
     void showSessionStatus();
     void slotSshConnectionError ( QString message, QString lastSessionError );
-    void slotSshServerAuthError ( int error, QString sshMessage );
+    void slotSshServerAuthError ( int error, QString sshMessage, SshMasterConnection* connection );
+    void slotSshServerAuthPassphrase ( SshMasterConnection* connection );
     void slotSshUserAuthError ( QString error );
     void slotSshConnectionOk();
     void slotServSshConnectionOk(QString server);
     void slotChangeKbdLayout(const QString& layout);
     void slotSyncX();
     void slotShutdownThinClient();
-    void slotReadApplications(bool result, QString output, SshProcess* proc );
+    void slotReadApplications(bool result, QString output, int pid );
 
 public slots:
     void slotConfig();
@@ -911,26 +963,26 @@ private slots:
     void slotTermSess();
     void slotNewSess();
     void slotGetBrokerAuth();
-    void slotGetBrokerSession(const QString& sinfo);
+    void slotGetBrokerSession();
     void slotCmdMessage ( bool result,QString output,
-                          SshProcess* );
+                          int );
     void slotListSessions ( bool result,QString output,
-                            SshProcess* );
+                            int );
     void slotRetSuspSess ( bool value,QString message,
-                           SshProcess* );
+                           int );
     void slotRetTermSess ( bool result,QString output,
-                           SshProcess* );
+                           int );
     void slotRetResumeSess ( bool result,QString output,
-                             SshProcess* );
+                             int );
     void slotTunnelFailed ( bool result,QString output,
-                            SshProcess* );
+                            int );
     void slotFsTunnelFailed ( bool result,QString output,
-                              SshProcess* );
+                              int );
     void slotSndTunnelFailed ( bool result,QString output,
-                               SshProcess* );
-    void slotCopyKey ( bool result,QString output,SshProcess* );
-    void slotTunnelOk();
-    void slotFsTunnelOk();
+                               int );
+    void slotCopyKey ( bool result,QString output,int );
+    void slotTunnelOk(int );
+    void slotFsTunnelOk(int );
     void slotProxyError ( QProcess::ProcessError err );
     void slotProxyFinished ( int result,QProcess::ExitStatus st );
     void slotProxyStderr();
@@ -940,13 +992,13 @@ private slots:
     void slotRestartProxy();
     void slotTestSessionStatus();
     void slotRetRunCommand ( bool result, QString output,
-                             SshProcess* );
+                             int );
     void slotGetServers ( bool result, QString output,
-                          SshProcess* );
+                          int );
     void slotListAllSessions ( bool result,QString output,
-                               SshProcess* );
+                               int );
     void slotRetExportDir ( bool result,QString output,
-                            SshProcess* );
+                            int );
     void slotResize();
     void slotExportDirectory();
     void slotExportTimer();
@@ -986,10 +1038,9 @@ private slots:
     void slotAttachProxyWindow();
     void slotEmbedIntoParentWindow();
     void slotEmbedWindow();
-    void slotStartParec ();
     void slotSndTunOk();
     void slotPCookieReady (	bool result,QString output,
-                            SshProcess* proc );
+                            int proc );
     void slotEmbedToolBar();
     void slotEmbedToolBarToolTip();
     void slotHideEmbedToolBarToolTip();
@@ -1026,7 +1077,9 @@ private:
     void printSshDError();
     void loadPulseModuleNativeProtocol();
     void initEmbedToolBar();
-    bool isServerRunning ( int port );
+#ifdef Q_OS_LINUX
+    void startDirectRDP();
+#endif
     void filterDesktops ( const QString& filter,
                           bool strict=false );
     void generateHostDsaKey();
