@@ -1,14 +1,20 @@
-//
-// C++ Implementation: sessionbutton
-//
-// Description:
-//
-//
-// Author: Oleksandr Shneyder <oleksandr.shneyder@obviously-nice.de>, (C) 2006
-//
-// Copyright: See COPYING file that comes with this distribution
-//
-//
+/**************************************************************************
+*   Copyright (C) 2005-2012 by Oleksandr Shneyder                         *
+*   o.shneyder@phoca-gmbh.de                                              *
+*                                                                         *
+*   This program is free software; you can redistribute it and/or modify  *
+*   it under the terms of the GNU General Public License as published by  *
+*   the Free Software Foundation; either version 2 of the License, or     *
+*   (at your option) any later version.                                   *
+*   This program is distributed in the hope that it will be useful,       *
+*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+*   GNU General Public License for more details.                          *
+*                                                                         *
+*   You should have received a copy of the GNU General Public License     *
+*   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
+***************************************************************************/
+
 #include "sessionbutton.h"
 #include "x2goclientconfig.h"
 
@@ -337,6 +343,7 @@ void SessionButton::redraw()
     cmdBox->addItem ( "KDE" );
     cmdBox->addItem ( "GNOME" );
     cmdBox->addItem ( "LXDE" );
+    cmdBox->addItem ( "XFCE" );
     cmdBox->addItem ( "UNITY" );
     cmdBox->addItem ( tr ( "RDP connection" ) );
     cmdBox->addItem ( tr ( "XDMCP" ) );
@@ -345,6 +352,7 @@ void SessionButton::redraw()
 
     cmdBox->addItems ( par->transApplicationsNames() );
 
+    bool directRDP=false;
     QPixmap cmdpix;
     if ( command=="KDE" )
     {
@@ -361,6 +369,11 @@ void SessionButton::redraw()
         cmdpix.load ( par->iconsPath ( "/16x16/unity.png" ) );
         cmdBox->setCurrentIndex ( UNITY );
     }
+    else if ( command == "XFCE" )
+    {
+        cmdpix.load ( par->iconsPath ( "/16x16/xfce.png" ) );
+        cmdBox->setCurrentIndex ( XFCE );
+    }
     else if ( command =="LXDE" )
     {
         cmdpix.load ( par->iconsPath ( "/16x16/lxde.png" ) );
@@ -374,6 +387,9 @@ void SessionButton::redraw()
     }
     else if ( command =="RDP" )
     {
+        if (st->setting()->value ( sid+"/directrdp",
+                                   ( QVariant ) false ).toBool())
+            directRDP=true;
         cmdpix.load ( par->iconsPath ( "/16x16/rdp.png" ) );
         cmdBox->setCurrentIndex ( RDP );
         command=tr ( "RDP connection" );
@@ -413,10 +429,11 @@ void SessionButton::redraw()
     geomBox->clear();
     geomBox->addItem ( tr ( "fullscreen" ) );
     uint displays=QApplication::desktop()->numScreens();
-    for (uint i=0;i<displays;++i)
-    {
-        geomBox->addItem ( tr( "Display " )+QString::number(i+1));
-    }
+    if (!directRDP)
+        for (uint i=0;i<displays;++i)
+        {
+            geomBox->addItem ( tr( "Display " )+QString::number(i+1));
+        }
 #ifndef Q_WS_HILDON
     geomBox->addItem ( "1440x900" );
     geomBox->addItem ( "1280x1024" );
@@ -432,7 +449,7 @@ void SessionButton::redraw()
     }
     else
         if (st->setting()->value ( sid+"/multidisp",
-                                   ( QVariant ) false ).toBool())
+                                   ( QVariant ) false ).toBool() && !directRDP)
         {
             uint disp=st->setting()->value ( sid+"/display",
                                              ( QVariant ) 1 ).toUInt();
@@ -468,6 +485,17 @@ void SessionButton::redraw()
 #endif
         }
 
+    if (directRDP)
+    {
+        geomBox->addItem ( tr("Maximum") );
+        if (st->setting()->value ( sid+"/maxdim",
+                                   ( QVariant ) false ).toBool())
+        {
+            geom->setText ( tr("Maximum") );
+            geomBox->setCurrentIndex ( geomBox->findText ( tr("Maximum") ));
+        }
+    }
+
 
     snd=st->setting()->value ( sid+"/sound", ( QVariant ) true ).toBool();
     if ( snd )
@@ -642,6 +670,11 @@ void SessionButton::slot_cmd_change ( const QString& command )
         newRootless=false;
         pix.load ( par->iconsPath ( "/16x16/unity.png" ) );
     }
+    else if ( command == "XFCE" )
+    {
+        newRootless=false;
+        pix.load ( par->iconsPath ( "/16x16/xfce.png" ) );
+    }
     else if ( command ==tr ( "Connection to local desktop" ) )
     {
         newRootless=false;
@@ -685,6 +718,11 @@ void SessionButton::slot_cmd_change ( const QString& command )
         cmd="UNITY";
         newRootless=false;
     }
+    if ( command=="xfce4-session" )
+    {
+        cmd="XFCE";
+        newRootless=false;
+    }
     if (command== tr("Published applications"))
     {
         published=true;
@@ -710,6 +748,13 @@ void SessionButton::slot_geom_change ( const QString& new_g )
     {
         st.setting()->setValue ( sid+"/fullscreen", ( QVariant ) true );
         st.setting()->setValue ( sid+"/multidisp", ( QVariant ) false );
+        st.setting()->setValue ( sid+"/maxdim", ( QVariant ) false );
+    }
+    else if ( new_g==tr ( "Maximum" ))
+    {
+        st.setting()->setValue ( sid+"/fullscreen", ( QVariant ) false );
+        st.setting()->setValue ( sid+"/multidisp", ( QVariant ) false );
+        st.setting()->setValue ( sid+"/maxdim", ( QVariant ) true );
     }
     else
         if (new_g.indexOf(tr("Display "))==0)
@@ -719,6 +764,7 @@ void SessionButton::slot_geom_change ( const QString& new_g )
             st.setting()->setValue ( sid+"/multidisp", ( QVariant ) true );
             st.setting()->setValue ( sid+"/display", ( QVariant ) g.toUInt());
             st.setting()->setValue ( sid+"/fullscreen", ( QVariant ) false );
+            st.setting()->setValue ( sid+"/maxdim", ( QVariant ) false );
         }
         else
         {
@@ -728,6 +774,7 @@ void SessionButton::slot_geom_change ( const QString& new_g )
 #endif
             st.setting()->setValue ( sid+"/fullscreen", ( QVariant ) false );
             st.setting()->setValue ( sid+"/multidisp", ( QVariant ) false );
+            st.setting()->setValue ( sid+"/maxdim", ( QVariant ) false );
             QStringList lst=new_geom.split ( 'x' );
             st.setting()->setValue ( sid+"/width", ( QVariant ) lst[0] );
             st.setting()->setValue ( sid+"/height", ( QVariant ) lst[1] );
