@@ -1,14 +1,20 @@
-//
-// C++ Implementation: settingswidget
-//
-// Description:
-//
-//
-// Author: Oleksandr Shneyder <oleksandr.shneyder@obviously-nice.de>, (C) 2009
-//
-// Copyright: See COPYING file that comes with this distribution
-//
-//
+/**************************************************************************
+*   Copyright (C) 2005-2012 by Oleksandr Shneyder                         *
+*   o.shneyder@phoca-gmbh.de                                              *
+*                                                                         *
+*   This program is free software; you can redistribute it and/or modify  *
+*   it under the terms of the GNU General Public License as published by  *
+*   the Free Software Foundation; either version 2 of the License, or     *
+*   (at your option) any later version.                                   *
+*   This program is distributed in the hope that it will be useful,       *
+*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+*   GNU General Public License for more details.                          *
+*                                                                         *
+*   You should have received a copy of the GNU General Public License     *
+*   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
+***************************************************************************/
+
 #include "settingswidget.h"
 #include "onmainwindow.h"
 #include <QBoxLayout>
@@ -26,10 +32,11 @@
 #include <QTimer>
 #include <QSplashScreen>
 #include "x2gologdebug.h"
+#include <QGridLayout>
 
 SettingsWidget::SettingsWidget ( QString id, ONMainWindow * mw,
                                  QWidget * parent, Qt::WindowFlags f )
-        : ConfigWidget ( id,mw,parent,f )
+    : ConfigWidget ( id,mw,parent,f )
 {
     multiDisp=(QApplication::desktop()->screenCount()>1);
 #ifdef Q_WS_HILDON
@@ -42,8 +49,8 @@ SettingsWidget::SettingsWidget ( QString id, ONMainWindow * mw,
     tabSettings->addTab ( sbgr,tr ( "Sound" ) );
 #else
     QGroupBox *dgb=new QGroupBox ( tr ( "&Display" ),this );
-    QGroupBox *kgb=new QGroupBox ( tr ( "&Keyboard" ),this );
-    QGroupBox *sbgr=new QGroupBox ( tr ( "Sound" ),this );
+    kgb=new QGroupBox ( tr ( "&Keyboard" ),this );
+    sbgr=new QGroupBox ( tr ( "Sound" ),this );
 #endif
     QVBoxLayout *dbLay = new QVBoxLayout ( dgb );
     QVBoxLayout  *sndLay=new QVBoxLayout ( sbgr );
@@ -61,11 +68,16 @@ SettingsWidget::SettingsWidget ( QString id, ONMainWindow * mw,
     custom=new QRadioButton ( tr ( "Window" ),dgb );
 #endif
     display=new QRadioButton ( tr ( "Use whole display" ),dgb );
-
+#ifdef Q_OS_LINUX
+    maxRes=new QRadioButton ( tr ( "Maximum available" ),dgb );
+#endif
     radio->addButton ( fs );
     radio->addButton ( custom );
     radio->setExclusive ( true );
     radio->addButton(display);
+#ifdef Q_OS_LINUX
+    radio->addButton(maxRes);
+#endif
     width=new QSpinBox ( dgb );
     height=new QSpinBox ( dgb );
     cbSetDPI=new QCheckBox ( tr ( "Set display DPI" ),dgb );
@@ -98,6 +110,9 @@ SettingsWidget::SettingsWidget ( QString id, ONMainWindow * mw,
     dwLay->addStretch();
 
     dispLay->addWidget(display);
+#ifdef Q_OS_LINUX
+    dispLay->addWidget(maxRes);
+#endif
     dispLay->addSpacing(15);
     dispLay->addWidget(lDisplay=new QLabel(tr("&Display:"),dgb));
     dispLay->addWidget(displayNumber=new QSpinBox(dgb));
@@ -123,10 +138,12 @@ SettingsWidget::SettingsWidget ( QString id, ONMainWindow * mw,
     dbLay->addLayout ( dwLay );
     dbLay->addLayout(dispLay);
     QFrame* dhl=new QFrame ( dgb );
+    hLine1=dhl;
     dhl->setFrameStyle ( QFrame::HLine | QFrame::Sunken );
     dbLay->addWidget ( dhl );
     dbLay->addLayout ( ddLay );
     dhl=new QFrame ( dgb );
+    hLine2=dhl;
     dhl->setFrameStyle ( QFrame::HLine | QFrame::Sunken );
     dbLay->addWidget ( dhl );
     dbLay->addWidget ( cbXinerama );
@@ -218,15 +235,42 @@ SettingsWidget::SettingsWidget ( QString id, ONMainWindow * mw,
     pulse->hide();
     esd->setChecked ( true );
 #endif
-
 #ifndef Q_WS_HILDON
     setLay->addWidget ( dgb );
     setLay->addWidget ( kgb );
     setLay->addWidget ( sbgr );
+#ifdef Q_OS_LINUX
+#ifdef CFGCLIENT
+    rdpBox=new QGroupBox ( tr ( "RDP Client" ),this );
+    setLay->addWidget ( rdpBox );
+    rRdesktop=new QRadioButton ("rdesktop",rdpBox );
+    rRdesktop->setChecked(true);
+    rXfreeRDP=new QRadioButton ( "xfreerdp",rdpBox);
+    QButtonGroup* rClient=new QButtonGroup(rdpBox);
+    rClient->addButton ( rRdesktop );
+    rClient->addButton ( rXfreeRDP );
+    rClient->setExclusive ( true );
+    QGridLayout *rdpLay=new QGridLayout(rdpBox);
+    rdpLay->addWidget(rRdesktop,0,0);
+    rdpLay->addWidget(rXfreeRDP,1,0);
+    rdpLay->addWidget(new QLabel(tr("Additional parameters:")),2,0);
+    rdpLay->addWidget(new QLabel(tr("Command line:")),3,0);
+    cmdLine=new QLineEdit(rdpBox);
+    cmdLine->setReadOnly(true);
+    params=new QLineEdit(rdpBox);
+    rdpLay->addWidget(cmdLine,4,0,1,2);
+    rdpLay->addWidget(params,2,1);
+    connect (rClient, SIGNAL(buttonClicked(int)), this, SLOT(updateCmdLine()));
+    connect (radio, SIGNAL(buttonClicked(int)), this, SLOT(updateCmdLine()));
+    connect (params, SIGNAL(textChanged(QString)), this, SLOT(updateCmdLine()));
+    connect (width, SIGNAL(valueChanged(int)), this, SLOT(updateCmdLine()));
+    connect (height, SIGNAL(valueChanged(int)), this, SLOT(updateCmdLine()));
+#endif //CFGCLIENT
+#endif //Q_OS_LINUX
 #else
     setLay->addWidget ( tabSettings );
 // 	cbClientPrint->hide();
-#endif
+#endif //Q_WS_HILDON
     setLay->addWidget ( cbClientPrint );
     setLay->addStretch();
 
@@ -267,6 +311,7 @@ SettingsWidget::SettingsWidget ( QString id, ONMainWindow * mw,
               SLOT ( slot_sndStartClicked() ) );
     connect ( cbDefSndPort,SIGNAL ( toggled ( bool ) ),this,
               SLOT ( slot_sndDefPortChecked ( bool ) ) );
+
     kbd->setChecked ( true );
     custom->setChecked ( true );
     readConfig();
@@ -277,12 +322,47 @@ SettingsWidget::~SettingsWidget()
 {
 }
 
+#ifdef Q_OS_LINUX
+void SettingsWidget::setDirectRdp(bool direct)
+{
+    cbClientPrint->setVisible(!direct);
+    kgb->setVisible(!direct);
+    sbgr->setVisible(!direct);
+    cbSetDPI->setVisible(!direct);
+    cbXinerama->setVisible(!direct);
+    display->setVisible(!direct);
+    maxRes->setVisible(direct);
+    DPI->setVisible(!direct);
+    lDisplay->setVisible(!direct);
+    displayNumber->setVisible(!direct);
+    pbIdentDisp->setVisible(!direct);
+    hLine1->setVisible(!direct);
+    hLine2->setVisible(!direct);
+    rdpBox->setVisible(direct);
+    if (direct)
+    {
+        if (display->isChecked())
+        {
+            display->setChecked(false);
+            custom->setChecked(true);
+        }
+    }
+    else
+    {
+        if (maxRes->isChecked())
+        {
+            maxRes->setChecked(false);
+            custom->setChecked(true);
+        }
+    }
+}
+#endif
 
 void SettingsWidget::slot_identDisplays()
 {
     pbIdentDisp->setEnabled(false);
     identWins.clear();
-    for (int i=0;i<QApplication::desktop()->screenCount();++i)
+    for (int i=0; i<QApplication::desktop()->screenCount(); ++i)
     {
         QMainWindow *mw=new QMainWindow(
             this, Qt::FramelessWindowHint|Qt::X11BypassWindowManagerHint|Qt::WindowStaysOnTopHint);
@@ -390,7 +470,7 @@ void SettingsWidget::slot_sndStartClicked()
     {
         lSndPort->setEnabled ( true );
         sbSndPort->setEnabled ( true );
-        cbDefSndPort->setEnabled ( true );
+        cbDefSndPort->setEnabled ( true &&sound->isChecked());
     }
     else
     {
@@ -457,6 +537,19 @@ void SettingsWidget::readConfig()
             displayNumber->setValue(1);
     }
 
+#ifdef Q_OS_LINUX
+#ifdef CFGCLIENT
+    maxRes->setChecked(st.setting()->value ( sessionId+"/maxdim", false).toBool());
+    QString client=st.setting()->value ( sessionId+"/rdpclient","rdesktop").toString();
+    if(client=="rdesktop")
+        rRdesktop->setChecked(true);
+    else
+        rXfreeRDP->setChecked(true);
+    params->setText(st.setting()->value ( sessionId+"/directrdpsettings","").toString());
+#endif
+#endif
+
+
     cbSetDPI->setChecked (
         st.setting()->value ( sessionId+"/setdpi",
                               ( QVariant ) mainWindow->getDefaultSetDPI() ).toBool() );
@@ -532,6 +625,9 @@ void SettingsWidget::readConfig()
     slot_sndToggled ( snd );
     slot_sndStartClicked();
 
+    if(!sound)
+        cbDefSndPort->setEnabled(false);
+
     cbClientPrint->setChecked ( st.setting()->value ( sessionId+"/print",
                                 true ).toBool() );
 }
@@ -575,6 +671,22 @@ void SettingsWidget::saveSettings()
                              ( QVariant ) display->isChecked() );
     st.setting()->setValue ( sessionId+"/display",
                              ( QVariant ) displayNumber->value() );
+
+#ifdef Q_OS_LINUX
+#ifdef CFGCLIENT
+    st.setting()->setValue ( sessionId+"/maxdim",
+                             ( QVariant ) maxRes->isChecked() );
+
+    if (rXfreeRDP->isChecked())
+        st.setting()->setValue ( sessionId+"/rdpclient",
+                                 ( QVariant ) "xfreerdp" );
+    else
+        st.setting()->setValue ( sessionId+"/rdpclient",
+                                 ( QVariant ) "rdesktop" );
+    st.setting()->setValue ( sessionId+"/directrdpsettings",
+                             ( QVariant ) params->text());
+#endif
+#endif
     st.setting()->setValue ( sessionId+"/height",
                              ( QVariant ) height->value() );
     st.setting()->setValue ( sessionId+"/dpi",
@@ -614,3 +726,46 @@ void SettingsWidget::saveSettings()
                              ( QVariant ) cbClientPrint->isChecked() );
     st.setting()->sync();
 }
+
+#ifdef Q_OS_LINUX
+void SettingsWidget::setServerSettings(QString server, QString port, QString user)
+{
+    this->server=server;
+    this->port=port;
+    this->user=user;
+    updateCmdLine();
+}
+
+void SettingsWidget::updateCmdLine()
+{
+#ifdef CFGCLIENT
+    QString client="xfreerdp";
+    QString userOpt;
+    if (user.length()>0)
+    {
+        userOpt=" -u ";
+        userOpt+=user;
+    }
+    if (rRdesktop->isChecked())
+    {
+        client="rdesktop";
+    }
+
+    QString grOpt;
+
+    if (fs->isChecked())
+    {
+        grOpt=" -f ";
+    }
+    if (maxRes->isChecked())
+    {
+        grOpt=" -D -g <maxW>x<maxH>";
+    }
+    if (custom->isChecked())
+    {
+        grOpt=" -g "+QString::number(width->value())+"x"+QString::number(height->value());
+    }
+    cmdLine->setText(client +" "+params->text()+ grOpt +userOpt+" -p <"+tr("password")+"> "+ server+":"+port );
+#endif
+}
+#endif
