@@ -1,23 +1,19 @@
-
-/***************************************************************************
- *   Copyright (C) 2005-2012 by Oleksandr Shneyder                         *
- *   oleksandr.shneyder@obviously-nice.de                                  *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
- ***************************************************************************/
+/**************************************************************************
+*   Copyright (C) 2005-2012 by Oleksandr Shneyder                         *
+*   o.shneyder@phoca-gmbh.de                                              *
+*                                                                         *
+*   This program is free software; you can redistribute it and/or modify  *
+*   it under the terms of the GNU General Public License as published by  *
+*   the Free Software Foundation; either version 2 of the License, or     *
+*   (at your option) any later version.                                   *
+*   This program is distributed in the hope that it will be useful,       *
+*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+*   GNU General Public License for more details.                          *
+*                                                                         *
+*   You should have received a copy of the GNU General Public License     *
+*   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
+***************************************************************************/
 
 #include "x2goclientconfig.h"
 #include "x2gologdebug.h"
@@ -26,6 +22,7 @@
 #include <stdio.h>
 #include "sshprocess.h"
 
+
 #include <QStringList>
 #include <QFile>
 #include <QDir>
@@ -36,30 +33,45 @@
 #include <math.h>
 
 #ifndef Q_OS_WIN
+#include <sys/socket.h> /* for socket(), connect(), send(), and recv() */
+#include <arpa/inet.h>  /* for sockaddr_in and inet_addr() */
 #include <arpa/inet.h>
 #include <netinet/tcp.h>
+#include <QtNetwork/qabstractsocket.h>
 #endif
 
 
 #include "onmainwindow.h"
 
-#undef DEBUG
-//#define DEBUG
 
-//#define SSH_DEBUG
+#define PROXYTUNNELPORT 44444
+
+#undef DEBUG
+// #define DEBUG
+
 #undef SSH_DEBUG
+// #define SSH_DEBUG
 
 static bool isLibSshInited=false;
 
-
-SshMasterConnection::SshMasterConnection ( QString host, int port, bool acceptUnknownServers, QString user,
-        QString pass, QString key,bool autologin, bool krblogin, QObject* parent ) : QThread ( parent )
+SshMasterConnection::SshMasterConnection (QObject* parent, QString host, int port, bool acceptUnknownServers, QString user,
+        QString pass, QString key, bool autologin, bool krblogin,
+        bool useproxy, ProxyType type, QString proxyserver, quint16 proxyport,
+        QString proxylogin, QString proxypassword, QString proxykey,
+        bool proxyautologin ) : QThread ( parent )
 {
 #if defined ( Q_OS_DARWIN )
     // Mac OS X provides only 512KB stack space for secondary threads.
     // As we put a 512KB buffer on the stack later on, we need a bigger stack space.
     setStackSize (sizeof (char) * 1024 * 1024 * 2);
 #endif
+    tcpProxySocket = NULL;
+    tcpNetworkProxy = NULL;
+    sshProxy= NULL;
+    sshProxyReady=false;
+    nextPid=0;
+
+    breakLoop=false;
     this->host=host;
     this->port=port;
     this->user=user;
@@ -67,6 +79,14 @@ SshMasterConnection::SshMasterConnection ( QString host, int port, bool acceptUn
     this->key=key;
     this->autologin=autologin;
     this->acceptUnknownServers=acceptUnknownServers;
+    this->useproxy=useproxy;
+    this->proxytype=type;
+    this->proxyautologin=proxyautologin;
+    this->proxykey=proxykey;
+    this->proxyserver=proxyserver;
+    this->proxyport=proxyport;
+    this->proxylogin=proxylogin;
+    this->proxypassword=proxypassword;
     reverseTunnel=false;
     mainWnd=(ONMainWindow*) parent;
     kerberos=krblogin;
@@ -77,16 +97,26 @@ SshMasterConnection::SshMasterConnection ( QString host, int port, bool acceptUn
         x2goDebug<<"starting ssh connection without kerberos authentication"<<endl;
 #endif
     kerberos=false;
+#ifdef DEBUG
+    x2goDebug<<"SshMasterConnection, instance "<<this<<" created";
+#endif
 }
 
-SshMasterConnection::SshMasterConnection ( QString host, int port, bool acceptUnknownServers, QString user,
-        QString pass, QString key, bool autologin,
+SshMasterConnection::SshMasterConnection (QObject* parent, ONMainWindow* mwd, QString host, int port, bool acceptUnknownServers,
+        QString user, QString pass, QString key,bool autologin,
         int remotePort, QString localHost, int localPort, SshProcess* creator,
-        QObject* parent, ONMainWindow* mwd ) : QThread ( parent )
+        bool useproxy, ProxyType type, QString proxyserver, quint16 proxyport,
+        QString proxylogin, QString proxypassword, QString proxykey,
+        bool proxyautologin, int localProxyPort) : QThread ( parent )
 {
 #if defined ( Q_OS_DARWIN )
     setStackSize (sizeof (char) * 1024 * 1024 * 2);
 #endif
+    tcpProxySocket = NULL;
+    tcpNetworkProxy = NULL;
+    sshProxy= NULL;
+    sshProxyReady=false;
+    breakLoop=false;
     this->host=host;
     this->port=port;
     this->user=user;
@@ -94,25 +124,145 @@ SshMasterConnection::SshMasterConnection ( QString host, int port, bool acceptUn
     this->key=key;
     this->autologin=autologin;
     this->acceptUnknownServers=acceptUnknownServers;
+    this->useproxy=useproxy;
+    this->proxyserver=proxyserver;
+    this->proxyport=proxyport;
+    this->proxylogin=proxylogin;
+    this->proxypassword=proxypassword;
+    this->proxytype=type;
+    this->proxyautologin=proxyautologin;
+    this->proxykey=proxykey;
+    this->localProxyPort=localProxyPort;
     reverseTunnelLocalHost=localHost;
     reverseTunnelLocalPort=localPort;
     reverseTunnelCreator=creator;
     reverseTunnel=true;
     reverseTunnelRemotePort=remotePort;
     mainWnd=mwd;
+#ifdef DEBUG
+    x2goDebug<<"SshMasterConnection, instance "<<this<<" created (reverse tunnel)";
+#endif
 }
 
+void SshMasterConnection::slotSshProxyConnectionOk()
+{
+#ifdef DEBUG
+    x2goDebug<<"sshproxy connected";
+#endif
+
+
+    localProxyPort=PROXYTUNNELPORT;
+    while ( ONMainWindow::isServerRunning ( localProxyPort ) )
+        ++localProxyPort;
+
+    sshProxy->startTunnel ( host, port, "localhost",localProxyPort,false,this, SLOT ( slotSshProxyTunnelOk(int)),
+                            SLOT ( slotSshProxyTunnelFailed(bool,QString,int)));
+
+}
+
+
+int SshMasterConnection::copyFile(const QString& src, const QString dst, QObject* receiver, const char* slotFinished)
+{
+    SshProcess* proc=new SshProcess(this, nextPid++);
+    if(receiver && slotFinished)
+    {
+        connect(proc, SIGNAL(sshFinished(bool,QString,int)), receiver, slotFinished);
+    }
+    proc->start_cp(src,dst);
+    processes<<proc;
+    return proc->pid;
+}
+
+int SshMasterConnection::executeCommand(const QString& command, QObject* receiver, const char* slotFinished)
+{
+    SshProcess* proc=new SshProcess(this, nextPid++);
+    if(receiver && slotFinished)
+    {
+        connect(proc, SIGNAL(sshFinished(bool,QString,int)), receiver, slotFinished);
+    }
+    proc->startNormal(command);
+    processes<<proc;
+    return proc->pid;
+
+}
+
+QString SshMasterConnection::getSourceFile(int pid)
+{
+    foreach (SshProcess* proc, processes)
+    {
+        if(proc->pid==pid)
+            return proc->getSource();
+    }
+    return QString ::null;
+}
+
+
+int SshMasterConnection::startTunnel(const QString& forwardHost, uint forwardPort, const QString& localHost, uint localPort, bool reverse,
+                                     QObject* receiver, const char* slotTunnelOk, const char* slotFinished)
+{
+    SshProcess* proc=new SshProcess(this, nextPid++);
+    if(receiver && slotFinished)
+    {
+        connect(proc, SIGNAL(sshFinished(bool,QString,int)), receiver, slotFinished);
+    }
+    if(receiver && slotTunnelOk)
+    {
+        connect(proc, SIGNAL(sshTunnelOk(int)), receiver, slotTunnelOk);
+    }
+    proc->startTunnel(forwardHost, forwardPort, localHost, localPort, reverse);
+    processes<<proc;
+    return proc->pid;
+}
+
+
+void SshMasterConnection::slotSshProxyConnectionError(QString err1, QString err2)
+{
+    breakLoop=true;
+    emit connectionError(tr("SSH proxy connection error"),err1+" "+err2);
+}
+
+void SshMasterConnection::slotSshProxyServerAuthError(int errCode, QString err, SshMasterConnection* con)
+{
+    emit serverAuthError(errCode, tr("SSH proxy connection error: ")+err, con);
+}
+
+void SshMasterConnection::slotSshProxyUserAuthError(QString err)
+{
+    breakLoop=true;
+    emit userAuthError(tr("SSH proxy connection error: ")+err);
+}
+
+
+void SshMasterConnection::slotSshProxyTunnelOk(int)
+{
+#ifdef DEBUG
+    x2goDebug<<"Ssh proxy tunnel established";
+#endif
+    sshProxyReady=true;
+}
+
+void SshMasterConnection::slotSshProxyTunnelFailed(bool ,  QString output,
+        int)
+{
+    breakLoop=true;
+    emit connectionError(tr("Failed to create SSH proxy tunnel"), output);
+}
+
+
 SshMasterConnection* SshMasterConnection::reverseTunnelConnection ( SshProcess* creator,
         int remotePort, QString localHost, int localPort )
 {
-    SshMasterConnection* con=new SshMasterConnection ( host,port,acceptUnknownServers,user,pass,
+    SshMasterConnection* con=new SshMasterConnection (this, mainWnd, host,port,acceptUnknownServers,user,pass,
             key,autologin, remotePort,localHost,
-            localPort,creator,this, mainWnd);
+            localPort,creator, useproxy, proxytype, proxyserver, proxyport, proxylogin,
+            proxypassword, proxykey, proxyautologin, localProxyPort );
     con->kerberos=kerberos;
 
     connect ( con,SIGNAL ( ioErr ( SshProcess*,QString,QString ) ),this,SIGNAL ( ioErr ( SshProcess*,QString,QString ) ) );
     connect ( con,SIGNAL ( stdErr ( SshProcess*,QByteArray ) ),this,SIGNAL ( stdErr ( SshProcess*,QByteArray ) ) );
     connect ( con,SIGNAL ( reverseListenOk ( SshProcess* ) ), this, SIGNAL ( reverseListenOk ( SshProcess* ) ) );
+    con->keyPhrase=keyPhrase;
+    con->keyPhraseReady=true;
     con->start();
     reverseTunnelConnectionsMutex.lock();
     reverseTunnelConnections.append ( con );
@@ -120,8 +270,46 @@ SshMasterConnection* SshMasterConnection::reverseTunnelConnection ( SshProcess*
     return con;
 }
 
+void SshMasterConnection::slotSshProxyServerAuthAborted()
+{
+    breakLoop=true;
+}
+
 void SshMasterConnection::run()
 {
+#ifdef DEBUG
+    x2goDebug<<"SshMasterConnection, instance "<<this<<" entering thread";
+#endif
+    if(useproxy && proxytype==PROXYSSH && !reverseTunnel)
+    {
+
+        sshProxy=new SshMasterConnection (0, proxyserver, proxyport,acceptUnknownServers,
+                                          proxylogin, proxypassword, proxykey, proxyautologin, kerberos, false);
+        connect ( sshProxy, SIGNAL ( connectionOk(QString) ), this, SLOT ( slotSshProxyConnectionOk() ) );
+
+        connect ( sshProxy, SIGNAL ( serverAuthError ( int,QString,SshMasterConnection* ) ),this,
+                  SLOT ( slotSshProxyServerAuthError ( int,QString, SshMasterConnection* ) ) );
+        connect ( sshProxy, SIGNAL ( needPassPhrase(SshMasterConnection*)),this,
+                  SIGNAL ( needPassPhrase(SshMasterConnection*)) );
+        connect ( sshProxy, SIGNAL ( serverAuthAborted()),this,
+                  SLOT ( slotSshProxyServerAuthAborted()) );
+        connect ( sshProxy, SIGNAL ( userAuthError ( QString ) ),this,SLOT ( slotSshProxyUserAuthError ( QString ) ) );
+        connect ( sshProxy, SIGNAL ( connectionError ( QString,QString ) ), this,
+                  SLOT ( slotSshProxyConnectionError ( QString,QString ) ) );
+
+        sshProxyReady=false;
+        sshProxy->start();
+
+        while(! sshProxyReady)
+        {
+            if(breakLoop)
+            {
+                quit();
+                return;
+            }
+            this->usleep(200);
+        }
+    }
     disconnectSessionFlag=false;
     if ( !isLibSshInited )
     {
@@ -164,6 +352,7 @@ void SshMasterConnection::run()
         quit();
         return;
     }
+
 #ifdef Q_OS_WIN
     ssh_options_set ( my_ssh_session, SSH_OPTIONS_SSH_DIR, (mainWnd->getHomeDirectory()+"/ssh").toAscii());
 #endif
@@ -171,10 +360,45 @@ void SshMasterConnection::run()
 
     ssh_options_set(my_ssh_session, SSH_OPTIONS_TIMEOUT, &timeout);
 
+    if (useproxy && proxytype == PROXYHTTP)
+    {
+        socket_t proxysocket = SSH_INVALID_SOCKET;
+
+        tcpNetworkProxy = new QNetworkProxy( QNetworkProxy::HttpProxy,
+                                             proxyserver, proxyport, proxylogin, proxypassword);
+        tcpProxySocket = new QTcpSocket();
+        tcpProxySocket->setProxy( *tcpNetworkProxy );
+        tcpProxySocket->connectToHost(host, port);
+        proxysocket = tcpProxySocket->socketDescriptor();
+        if (!tcpProxySocket->waitForConnected(30000))
+        {
+            QString message=tr ( "Can not connect to proxy server" );
+            x2goDebug<<message<<endl;
+            emit connectionError ( "Proxy", message );
+            ssh_free ( my_ssh_session );
+            quit();
+            return;
+        }
+#ifdef DEBUG
+        x2goDebug <<  "Created HTTP proxy socket " << proxysocket << endl;
+#endif
+        ssh_options_set( my_ssh_session, SSH_OPTIONS_FD, &proxysocket);
+        ssh_set_fd_toread( my_ssh_session);
+#ifdef DEBUG
+        x2goDebug<<"Connected to HTTP proxy server " << proxyserver << ":"
+                 << proxyport <<endl;
+#endif
+    }
+
     if ( !sshConnect() )
     {
+        if(disconnectSessionFlag)
+        {
+            x2goDebug<<"session already disconnected, exiting"<<endl;
+            return;
+        }
         QString err=ssh_get_error ( my_ssh_session );
-        QString message=tr ( "Can not connect to " ) +host+QString::number ( port );
+        QString message=tr ( "Can not connect to " ) +host+":"+QString::number ( port );
         x2goDebug<<message<<" - "<<err;
         emit connectionError ( message, err );
         if ( reverseTunnel )
@@ -183,17 +407,45 @@ void SshMasterConnection::run()
         quit();
         return;
     }
+    if(disconnectSessionFlag)
+    {
+        x2goDebug<<"session already disconnected, exiting"<<endl;
+        return;
+    }
     QString errMsg;
     int state=serverAuth ( errMsg );
     if ( state != SSH_SERVER_KNOWN_OK )
     {
-        emit serverAuthError ( state,errMsg );
+        if(disconnectSessionFlag)
+        {
+            x2goDebug<<"session already disconnected, exiting"<<endl;
+            return;
+        }
+        writeHostKey=writeHostKeyReady=false;
+        emit serverAuthError ( state,errMsg, this );
+        for(;;)
+        {
+            this->usleep(100);
+            writeHostKeyMutex.lock();
+            if(writeHostKeyReady)
+            {
+                if(writeHostKey)
+                    ssh_write_knownhost(my_ssh_session);
+                writeHostKeyMutex.unlock();
+                break;
+            }
+            writeHostKeyMutex.unlock();
+        }
         ssh_disconnect ( my_ssh_session );
         ssh_free ( my_ssh_session );
-        quit();
         return;
     }
 
+    if(disconnectSessionFlag)
+    {
+        x2goDebug<<"session already disconnected, exiting"<<endl;
+        return;
+    }
     ssh_options_set ( my_ssh_session, SSH_OPTIONS_USER, user.toAscii() );
 #ifdef Q_OS_WIN
     ssh_options_set ( my_ssh_session, SSH_OPTIONS_SSH_DIR, (mainWnd->getHomeDirectory()+"/ssh").toAscii());
@@ -205,6 +457,11 @@ void SshMasterConnection::run()
 
     if ( userAuth() )
     {
+        if(disconnectSessionFlag)
+        {
+            x2goDebug<<"session already disconnected, exiting"<<endl;
+            return;
+        }
 #ifdef DEBUG
         x2goDebug<<"user auth OK\n";
 #endif
@@ -212,6 +469,11 @@ void SshMasterConnection::run()
     }
     else
     {
+        if(disconnectSessionFlag)
+        {
+            x2goDebug<<"session already disconnected, exiting"<<endl;
+            return;
+        }
         QString err;
         if (!kerberos)
             err=ssh_get_error ( my_ssh_session );
@@ -240,6 +502,11 @@ void SshMasterConnection::run()
     {
         if ( channel_forward_listen ( my_ssh_session, NULL, reverseTunnelRemotePort,  NULL ) !=SSH_OK )
         {
+            if(disconnectSessionFlag)
+            {
+                x2goDebug<<"session already disconnected, exiting"<<endl;
+                return;
+            }
             QString err=ssh_get_error ( my_ssh_session );
             QString message=tr ( "channel_forward_listen failed" );
             x2goDebug<<message<<" - "<<err;
@@ -261,8 +528,27 @@ void SshMasterConnection::run()
 
 SshMasterConnection::~SshMasterConnection()
 {
-    /*    ssh_disconnect(my_ssh_session);
-        ssh_free(my_ssh_session);*/
+
+    disconnectFlagMutex.lock();
+    disconnectSessionFlag=true;
+    disconnectFlagMutex.unlock();
+#ifdef DEBUG
+    x2goDebug<<"SshMasterConnection, instance "<<this<<" waiting for thread to finish";
+#endif
+    if(!reverseTunnel)
+        wait(15000);
+    else
+        wait(5000);
+#ifdef DEBUG
+    x2goDebug<<"SshMasterConnection, instance "<<this<<" thread finished";
+#endif
+    for(int i=processes.count()-1; i>=0; --i)
+    {
+        delete processes[i];
+    }
+#ifdef DEBUG
+    x2goDebug<<"SshMasterConnection, instance "<<this<<" SshMasterConnection, instance  SshMasterConnection(0x7fce7c008aa0) deleted";
+#endif
 }
 
 
@@ -286,16 +572,51 @@ bool SshMasterConnection::sshConnect()
 {
     int rc;
     QByteArray tmpBA = host.toLocal8Bit();
-    ssh_options_set ( my_ssh_session, SSH_OPTIONS_HOST, tmpBA.data() );
-    ssh_options_set ( my_ssh_session, SSH_OPTIONS_PORT, &port );
+    if(useproxy && proxytype==PROXYSSH)
+    {
+#ifdef Q_OS_WIN
+        ssh_options_set ( my_ssh_session, SSH_OPTIONS_HOST, "127.0.0.1" );
+#else
+        ssh_options_set ( my_ssh_session, SSH_OPTIONS_HOST, "localhost" );
+#endif
+        ssh_options_set ( my_ssh_session, SSH_OPTIONS_PORT, &localProxyPort );
+
+    }
+    else
+    {
+        ssh_options_set ( my_ssh_session, SSH_OPTIONS_HOST, tmpBA.data() );
+        ssh_options_set ( my_ssh_session, SSH_OPTIONS_PORT, &port );
+    }
     rc = ssh_connect ( my_ssh_session );
     if ( rc != SSH_OK )
+    {
         return false;
-    return true;
+    }
+//set values for remote host for proper server authentication
+    if(useproxy && proxytype==PROXYSSH)
+    {
+        ssh_options_set ( my_ssh_session, SSH_OPTIONS_HOST, tmpBA.data() );
+        ssh_options_set ( my_ssh_session, SSH_OPTIONS_PORT, &port );
 
+    }
+    return true;
 }
 
 
+void SshMasterConnection::writeKnownHosts(bool write)
+{
+    writeHostKeyMutex.lock();
+    writeHostKeyReady=true;
+    writeHostKey=write;
+    if(!write)
+    {
+        breakLoop=true;
+        emit serverAuthAborted();
+    }
+    writeHostKeyMutex.unlock();
+
+}
+
 
 int SshMasterConnection::serverAuth ( QString& errorMsg )
 {
@@ -326,7 +647,7 @@ int SshMasterConnection::serverAuth ( QString& errorMsg )
 
     case SSH_SERVER_KNOWN_CHANGED:
         hexa = ssh_get_hexa ( hash, hlen );
-        errorMsg=hexa;
+        errorMsg=host+":"+QString::number(port)+" - "+hexa;
         free ( hexa );
         break;
     case SSH_SERVER_FOUND_OTHER:
@@ -336,7 +657,7 @@ int SshMasterConnection::serverAuth ( QString& errorMsg )
         if ( !acceptUnknownServers )
         {
             hexa = ssh_get_hexa ( hash, hlen );
-            errorMsg=hexa;
+            errorMsg=host+":"+QString::number(port)+" - "+hexa;
             free ( hexa );
             break;
         }
@@ -345,7 +666,7 @@ int SshMasterConnection::serverAuth ( QString& errorMsg )
         break;
 
     case SSH_SERVER_ERROR:
-        errorMsg=ssh_get_error ( my_ssh_session );
+        errorMsg=host+":"+QString::number(port)+" - "+ssh_get_error ( my_ssh_session );
         break;
     }
     free ( hash );
@@ -371,7 +692,35 @@ bool SshMasterConnection::userAuthWithPass()
 
 bool SshMasterConnection::userAuthAuto()
 {
-    int rc = ssh_userauth_autopubkey ( my_ssh_session, NULL );
+    int rc = ssh_userauth_autopubkey ( my_ssh_session, "" );
+    int i=0;
+    while(rc != SSH_AUTH_SUCCESS)
+    {
+        if(!reverseTunnel)
+        {
+            keyPhraseReady=false;
+            emit needPassPhrase(this);
+            for(;;)
+            {
+                bool ready=false;
+                this->usleep(200);
+                keyPhraseMutex.lock();
+                if(keyPhraseReady)
+                    ready=true;
+                keyPhraseMutex.unlock();
+                if(ready)
+                    break;
+            }
+        }
+        if(keyPhrase==QString::null)
+            break;
+        rc = ssh_userauth_autopubkey ( my_ssh_session, keyPhrase.toAscii() );
+        if(i++==2)
+        {
+            break;
+        }
+    }
+
     if ( rc != SSH_AUTH_SUCCESS )
     {
         QString err=ssh_get_error ( my_ssh_session );
@@ -384,6 +733,15 @@ bool SshMasterConnection::userAuthAuto()
     return true;
 }
 
+void SshMasterConnection::setKeyPhrase(QString phrase)
+{
+    keyPhraseMutex.lock();
+    keyPhrase=phrase;
+    keyPhraseReady=true;
+    keyPhraseMutex.unlock();
+}
+
+
 bool SshMasterConnection::userAuthWithKey()
 {
 #ifdef DEBUG
@@ -409,7 +767,34 @@ bool SshMasterConnection::userAuthWithKey()
 #endif
     }
 
-    ssh_private_key prkey=privatekey_from_file(my_ssh_session, keyName.toAscii(), 0, pass.toAscii());
+    ssh_private_key prkey=privatekey_from_file(my_ssh_session, keyName.toAscii(), 0,"");
+    int i=0;
+    while(!prkey)
+    {
+        if(!reverseTunnel)
+        {
+            keyPhraseReady=false;
+            emit needPassPhrase(this);
+            for(;;)
+            {
+                bool ready=false;
+                this->usleep(200);
+                keyPhraseMutex.lock();
+                if(keyPhraseReady)
+                    ready=true;
+                keyPhraseMutex.unlock();
+                if(ready)
+                    break;
+            }
+        }
+        if(keyPhrase==QString::null)
+            break;
+        prkey=privatekey_from_file(my_ssh_session, keyName.toAscii(), 0,keyPhrase.toAscii());
+        if(i++==2)
+        {
+            break;
+        }
+    }
     if (!prkey)
     {
 #ifdef DEBUG
@@ -498,6 +883,7 @@ void SshMasterConnection::addChannelConnection ( SshProcess* creator, int sock,
 
 void SshMasterConnection::addChannelConnection ( SshProcess* creator, QString cmd )
 {
+
     ChannelConnection con;
     con.channel=NULL;
     con.sock=-1;
@@ -521,17 +907,11 @@ void SshMasterConnection::addCopyRequest ( SshProcess* creator, QString src, QSt
     copyRequestMutex.unlock();
 }
 
-void SshMasterConnection::disconnectSession()
-{
-    disconnectFlagMutex.lock();
-    disconnectSessionFlag=true;
-    disconnectFlagMutex.unlock();
-}
 
 void SshMasterConnection::copy()
 {
 
-    for ( int i=copyRequests.size()-1;i>=0;--i )
+    for ( int i=copyRequests.size()-1; i>=0; --i )
     {
         QStringList lst=copyRequests[i].dst.split ( "/" );
         QString dstFile=lst.last();
@@ -603,6 +983,61 @@ void SshMasterConnection::channelLoop()
 {
     forever
     {
+        disconnectFlagMutex.lock();
+        bool disconnect=disconnectSessionFlag;
+        disconnectFlagMutex.unlock();
+
+        if ( disconnect )
+        {
+#ifdef DEBUG
+            x2goDebug<<"Disconnecting..."<<endl;
+#endif
+
+            if (useproxy && proxytype==PROXYSSH&&sshProxy)
+            {
+                delete sshProxy;
+                sshProxy=0;
+            }
+
+            reverseTunnelConnectionsMutex.lock();
+#ifdef DEBUG
+            x2goDebug<<"Deleting reverse tunnel connections"<<endl;
+#endif
+            for ( int i=reverseTunnelConnections.size()-1; i>=0; --i)
+            {
+                delete reverseTunnelConnections[i];
+            }
+            reverseTunnelConnectionsMutex.unlock();
+
+            channelConnectionsMutex.lock();
+#ifdef DEBUG
+            x2goDebug<<"Deleting channel connections"<<endl;
+#endif
+            for ( int i=0; i<channelConnections.size(); ++i )
+            {
+                finalize ( i );
+            }
+            channelConnectionsMutex.unlock();
+#ifdef DEBUG
+            x2goDebug<<"Disconnect session"<<endl;
+#endif
+            ssh_disconnect ( my_ssh_session );
+            ssh_free ( my_ssh_session );
+
+#ifdef DEBUG
+            x2goDebug<<"Delete sockets"<<endl;
+#endif
+            if (tcpProxySocket != NULL)
+                delete tcpProxySocket;
+            if (tcpNetworkProxy != NULL)
+                delete tcpNetworkProxy;
+#ifdef DEBUG
+            if ( !reverseTunnel )
+                x2goDebug<<"All channels closed, session disconnected, quiting session loop"<<endl;
+#endif
+            quit();
+            return;
+        }
         copyRequestMutex.lock();
         if ( copyRequests.size() >0 )
             copy();
@@ -669,42 +1104,6 @@ void SshMasterConnection::channelLoop()
         int retval;
         int maxsock=-1;
 
-        disconnectFlagMutex.lock();
-        bool disconnect=disconnectSessionFlag;
-        disconnectFlagMutex.unlock();
-
-        if ( disconnect )
-        {
-#ifdef DEBUG
-            if ( !reverseTunnel )
-                x2goDebug<<"Disconnecting..."<<endl;
-#endif
-            reverseTunnelConnectionsMutex.lock();
-            for ( int i=0;i<reverseTunnelConnections.size();++i )
-            {
-                reverseTunnelConnections[i]->disconnectSession();
-                reverseTunnelConnections[i]->wait ( 10000 );
-
-            }
-            reverseTunnelConnectionsMutex.unlock();
-
-            channelConnectionsMutex.lock();
-            for ( int i=0;i<channelConnections.size();++i )
-            {
-                finalize ( i );
-            }
-            channelConnectionsMutex.unlock();
-
-            ssh_disconnect ( my_ssh_session );
-            ssh_free ( my_ssh_session );
-#ifdef DEBUG
-            if ( !reverseTunnel )
-                x2goDebug<<"All channels closed, session disconnected, quiting session loop"<<endl;
-#endif
-            quit();
-            return;
-        }
-
 
         channelConnectionsMutex.lock();
         if ( channelConnections.size() <=0 )
@@ -720,7 +1119,7 @@ void SshMasterConnection::channelLoop()
 
         FD_ZERO ( &rfds );
 
-        for ( int i=0;i<channelConnections.size();++i )
+        for ( int i=0; i<channelConnections.size(); ++i )
         {
             int tcpSocket=channelConnections.at ( i ).sock;
             if ( tcpSocket>0 )
@@ -804,7 +1203,7 @@ void SshMasterConnection::channelLoop()
 //         x2goDebug<<"select exited"<<endl;
 
         channelConnectionsMutex.lock();
-        for ( int i=channelConnections.size()-1;i>=0;--i )
+        for ( int i=channelConnections.size()-1; i>=0; --i )
         {
             int tcpSocket=channelConnections.at ( i ).sock;
             ssh_channel channel=channelConnections.at ( i ).channel;
@@ -936,9 +1335,13 @@ void SshMasterConnection::finalize ( int item )
     }
     if ( tcpSocket>0 )
     {
+#ifndef Q_OS_WIN
+        shutdown(tcpSocket, SHUT_RDWR);
+#endif
         close ( tcpSocket );
     }
     SshProcess* proc=channelConnections[item].creator;
     channelConnections.removeAt ( item );
     emit channelClosed ( proc );
 }
+
